package main

import (
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"r2go/api"
	"r2go/tools"
	"r2go/utils"
	"strings"

	"github.com/fatih/color"
)

var version string = "0.0.1"

func downloadMod(pkg string) {
	buffer := fmt.Sprint(" > ", tools.ExposeModString(pkg), "... ")
	stat, ov := tools.DownloadMod(tools.ExposeModString(pkg))

	green := color.New(color.FgGreen).SprintFunc()
	magenta := color.New(color.FgMagenta).SprintFunc()
	blue := color.New(color.FgBlue).SprintFunc()

	if stat == 1 {
		buffer += green("up to date!")
	} else if stat == 2 {
		buffer += fmt.Sprintf("%s %s", magenta("updating to version"), ov)
	} else {
		buffer += blue("downloaded")
	}

	fmt.Println(buffer)
}

func removeMod(pkg string) {
	blue := color.New(color.FgBlue).SprintFunc()
	red := color.New(color.FgRed).SprintFunc()

	buffer := fmt.Sprint(" > ", tools.ExposeModString(pkg), "... ")

	if tools.RemoveMod(tools.ExposeModString(pkg)) == 1 {
		buffer += red("not installed")
	} else {
		buffer += blue("uninstalled")
	}

	fmt.Println(buffer)
}

func main() {

	logFile, err := os.OpenFile(utils.SystemInfo.TmpDir+"/r2mod-go.log", os.O_RDWR|os.O_CREATE|os.O_APPEND, 0644)
	utils.CheckErr(err)

	defer logFile.Close()
	log.SetOutput(logFile)

	// Setup CLI switch

	var operation string
	if len(os.Args) >= 2 {
		operation = os.Args[1]
	} else {
		operation = "help"
	}

	// Color
	white := color.New(color.FgWhite).SprintFunc()

	// CLI Operations

	switch operation {
	case "help":
		fmt.Println("> Commands:")
		fmt.Println(" > info -- get info about a mod")
		fmt.Println(" > install, ins, i -- install mod(s)")
		fmt.Println(" > remove, rm, r -- remove mod(s)")
		fmt.Println(" > list, ls, li -- list mods")
		fmt.Println(" > search, s, find -- search mods")
		fmt.Println(" > update, upgrade, up -- update mods and API cache")
		fmt.Println(" > toggle, tm, togglemods -- toggle mods")
		fmt.Println(" > pull -- forcefully update API cache, not mods")
		fmt.Println(" > filter -- remove version numbers using regex from stdin")
		fmt.Println(" > version, ver, v -- print information about the program")
	case "info":
		api.InitAPI()
		if len(os.Args) <= 2 {
			fmt.Println("Usage: r2go info <dependency string>")
		} else {
			var selectedmod api.Mod = api.GetModData(os.Args[2])
			color.Cyan("Mod Info: %s", os.Args[2])
			fmt.Println("  Name: " + selectedmod.Versions[0].FullName)
			fmt.Println("  Desc: " + selectedmod.Versions[0].Description)
			fmt.Println("  Version: " + selectedmod.Versions[0].VersionNumber)
			fmt.Println("  Download URL: " + selectedmod.Versions[0].DownloadURL)
		}
	case "install", "ins", "i":
		api.CheckAPICache()
		api.InitAPI()
		fmt.Println("> Downloading mods...")
		for _, m := range os.Args[2:] {
			downloadMod(m)
		}
		color.Green("> Complete!")
	case "remove", "rm", "r":
		for _, m := range os.Args[2:] {
			removeMod(m)
		}
	case "list", "ls", "li":
		files, err := ioutil.ReadDir(utils.SystemInfo.PluginDir)
		if err != nil {
			log.Fatal(err)
		}

		for i, f := range files {
			color.Blue("%3v %s", i, white(f.Name()))
		}
	case "update", "upgrade", "up":
		color.Cyan("> Checking API Cache")
		api.InitAPI()
		if api.CheckAPICache() == 0 {
			color.Green("> API Cache is up to date!")
		} else {
			color.Red("> API Cache out of date!")
			api.UpdateAPICache()
		}

		files, err := ioutil.ReadDir(utils.SystemInfo.PluginDir)
		if err != nil {
			log.Fatal(err)
		}

		for _, f := range files {
			downloadMod(f.Name())
		}

		color.Green("> Complete!")
	case "pull":
		api.InitAPI()
		color.Cyan("> Checking API Cache")
		if api.CheckAPICache() == 0 {
			color.Green("> API Cache is up to date!")
		} else {
			color.Red("> API Cache out of date!")
			api.UpdateAPICache()
		}
	case "toggle", "tm", "togglemods":
		if tools.ToggleMods() {
			color.Cyan("> Mods Enabled")
		} else {
			color.Cyan("> Mods Enabled")
		}
	case "filter":
		fmt.Println(tools.ExposeModString(os.Args[2]))
	case "version", "ver", "v":
		color.Magenta("> r2go", version)
		fmt.Println(" > github.com/endigma442")
	case "search", "s", "find":
		api.InitAPI()

		query := strings.Join(os.Args[2:], "-")

		blue := color.New(color.FgBlue).SprintFunc()

		fmt.Printf("%s %s\n", "Querying API for mods matching:", blue(query))

		for _, mod := range api.SearchMods(query) {
			fmt.Println(" -", mod.Target)
		}
	default:
		fmt.Println("Unknown command, use 'r2go help' for a list of commands.")
	}
}
